function  results= FDMa(X0, parameters, savefilename)
%function results= FDMa(X0, parameters, savefilename)
%
% FREQUENCY-DOMAIN MULTIVARIATE ANALYSIS (c) 2008 
% http://www.fdm.uni-freiburg.de
%
% Input:
%   X0                  : Stationary signal portion, size(X0)== [Norig, D], 
%                         N: signal length, D: dimension
%   savefilename        : a unique name for the given signal X0
%
%   parameters          : structure of parameters
% .silent                 Switch on/off output messages
% .resultspath            Path for result files, in thes folder a folder <savefilename will be created>
% .latexpath              Path for latex file. This path is relative from the location of the mother-latex file
%
%------Analysis parameters---------------------------------------
% .Fs0                    Sampling frequency in Hertz
% .Ldo                    Subsampling factor
% .f0                     Frequencies at which the graph will be computed
% .siglev0                Overall significance level
% .P0                     Override order selection. Give VAR order P
% .h0Hz                   Override smoother design. Give half smoother width in Hertz
%
%------Plot partameters-------------------------------------------
% .plotting               Switch plotting on/off
% .deltaT, .deltaT        Tick spacing
% .plot_diag              Switch on/off plotting of main diagonal
% .plot_zoom              Switch on/off plotting of zoomed .pdf
% .plot_fig               Switch on/off plotting of .fig
% .plot_diagnosis         Switch on/off plotting of some diagnostic info (taper, smoother, etc.)
% .position_SIG           Screen positions for the figures
% .position_SMP
% .position_VAR
% .position_COH
% .position_PDC
%
%------Graph parameters--------------------------------------------
% .graphing               Switch graphing on/off
% .filetype               Graph output file type
% .graphstyle             'circo' or 'dot' or 'neato'
% .labelon                Switch labels on/off
% .edge_weight_factor     Edge weight factor
% .node_labels            Node labels
% .node_colors            Node colors
% .node_slot              Further node parameters (not tested)
% .node_positions         Manual node positions
%
%------LaTeX parameters--------------------------------------------      
% .latexing               Switch latexing on/off
%
% Output:
%   results             : structure of parameters
% .SMP                    Smoothed and tapered periodogram matrix, size(SMP)==[D, D, N]
% .VAR                    VAR spectral matrix, size(VAR)==[D, D, N]
% .COH                    Coherence & partial coherence matrix, size(COH)==[D, D, N]
% .COHsig                 COH & partial coherence significance niveau (single number)
% .PDC                    Partial directed coherence (upper+lower), VAR (diagonal), size(PDC)==[D, D, N]
% .PDCsig                 PDC significance niveau, size(PDCsig)==[D, D, N]
% .Ahat                   Estimated VAR parameters, size(Aat)==[D, Phat*D]
% .Chat                   Estimated noise covariance, size(Chat)==[D, D]
% .Phat                   Estimated VAR order (single number)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Check for savefilename, if empty create standard name
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(~exist('savefilename', 'var'))
   savefilename= 'FDMa_output';
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Extract all parameters from parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
vars= genvarname(fieldnames(parameters));
for i= 1:length(vars)
   eval([vars{i} '= parameters.(vars{i});'])
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Only plotting or analysis plus plotting
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(~analyzing)
   cd(resultspath)
   cd(savefilename);
   load(savefilename)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Posible create output directory
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else
   if(~isempty(savefilename))
      cd(resultspath)
      %if(exist(['./' savefilename], 'dir')~=7)
      if(exist([savefilename], 'dir')~=7)
         eval(sprintf('mkdir %s', savefilename));
      end
      cd(savefilename);
      if(~silent)
         disp(sprintf('\nFDMa started to analyze %s, results will be written to %s', savefilename, pwd))
      end
   else
      if(~silent)
         disp(sprintf('\nFDMa started. No results will be written'))
      end
   end

   [Norig, D]= size(X0);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   if(Ldo>1)%% Downsampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      if(rem(Norig/Ldo, 2))
         Nnew= ceil(Norig/Ldo/2)*2*Ldo;
         X0= [X0; zeros(Nnew-Norig, D)];
      end
      [Norig, D]= size(X0);
      if(Ldo>10)
         ldos= factor(Ldo);
      else
         ldos= Ldo;
      end
      Xdown= [];
      for d= 1:D
         x= X0(:, d);
         ffs= Fs;
         for ll= 1:length(ldos)
            y2= downsam(x, ffs/ldos(ll), ffs);
            ffs= ffs/ldos(ll);
         end
         Xdown= [Xdown y2];
      end
      parameters.Fs= Fs/Ldo;
      parameters.N= Norig/Ldo;
      X= Xdown;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   else%if(Ldo>1)%% Downsampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      if(rem(length(X0), 2))
         X= [X0; zeros(1, D)];
      else
         X= X0;
      end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   end%if(Ldo>1)%% Downsampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   [N, D]= size(X);
   parameters.N= N;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Parameters for multiple testing
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   if(isempty(f0))
% Number of tests is one, f0 will be estimated
      K= 1;
% Half smoothing window length (default)
      parameters.h= 8;
   else
% Delete f0's which are above the half sampling frequency
      f0= setdiff(f0, f0(f0>parameters.Fs/2));
% Delete f0's which are below zero
      f0= setdiff(f0, f0(f0<0));
% Number of tests
      parameters.f0= f0;
      K= length(f0);
% Half smoothing window length
      parameters.h= min(ceil(N/32), ceil(min(diff([0; f0(:)]))/2/parameters.Fs*N));
   end

% Significance level for all tests
   parameters.siglev= siglev0/K;
% Minimum search range for VAR order estimation
   parameters.Pmin= K;
% Maxiumum search range for VAR order estimation
   parameters.Pmax= max(parameters.Pmin, 50);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Override order selection
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   if(P0)
      parameters.Pmin= P0;
      parameters.Pmax= P0;
   end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Override smoother design
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   if(h0Hz)
      parameters.h= floor(parameters.h0Hz/parameters.Fs*parameters.N);
   end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set a triangular smoothing window 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   if(parameters.h==0)
      parameters.win= 1;
   else
      parameters.win= 1/parameters.h-abs((-parameters.h:parameters.h)')/parameters.h.^2;
   end
   parameters.win= parameters.win/sum(parameters.win);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Set a raised-cosine taper 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Taper roll-off
   rolloff= ceil(N/16);
   parameters.taperwin= ones(N/2, 1);
   parameters.taperwin(1:2*rolloff+1)= (1-cos(pi/(2*rolloff+1)*(0:2*rolloff)))/2;
   parameters.taperwin= [parameters.taperwin; flipud(parameters.taperwin)];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Call the estimation FDMa_analyze.m
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   switch(FDMaversion)
      case 1
         [results, parameters]= FDMa_analyze(X, parameters);
      case 2
         disp('FDMa2 started!')
         [results, parameters]= FDMa2_analyze(X, parameters);
   end
   
   if(~isempty(savefilename))
      save(savefilename, 'results', 'parameters')
      if(~silent)
         disp(sprintf('analyzed %s and saved to %s: %s', savefilename, savefilename, results.string))
      end
   else
      if(~silent)
         disp(sprintf('analyzed but not saved: %s', results.string))
      end
   end
end%if(analyzing)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Plot the results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(plotting)
   FDMa_plot(parameters, savefilename);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Compute and plot frequency-resloved interaction graphs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(graphing)
   FDMa_graph(parameters, savefilename);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Generate a nice LaTeX figure
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(latexing)
   FDMa_latex(parameters, savefilename);
end
